#------------------------------------------------------------------------------
#
# Project: prism view server
# Authors: Fabian Schindler <fabian.schindler@eox.at>
#
#------------------------------------------------------------------------------
# Copyright (C) 2020 EOX IT Services GmbH <https://eox.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies of this Software or works derived from this Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#-----------------------------------------------------------------------------


from os.path import dirname, join
from datetime import datetime
from dateutil import tz

from ingestor.browse_report import parse_browse_report


TEST_DATA_DIR = join(dirname(dirname(__file__)), 'tests/data')


def test_parse_footprint_browse():
    with open(join(TEST_DATA_DIR, 'footprint_browse.xml')) as f:
        browse_report = parse_browse_report(f)

    assert browse_report == {
        'responsible_org_name': 'Generated by Eoli 2 ngEO Converter V1.2.0',
        'date_time': datetime(2013, 9, 25, 14, 54, 38, 0, tz.UTC),
        'browse_type': 'SAR',
        'browses': [{
            'type': 'footprint_browse',
            'browse_identifier': 'ERS-2-11040113373745-1507.SAR_IM0_0P.BP',
            'filename': 'ERS-2-11040113373745-1507.SAR_IM0_0P.BP.jpg',
            'image_type': 'Jpeg',
            'reference_system_identifier': 'EPSG:4326',
            'footprint': {
                'col_row_list': [
                    (0, 0),
                    (500, 0),
                    (500, 250),
                    (0, 250),
                    (0, 0),
                ],
                'coord_list': [
                    (83.66, 42.31),
                    (84.53, 42.42),
                    (84.48, 51.28),
                    (83.61, 50.32),
                    (83.66, 42.31),
                ]
            },
            'start_time': datetime(2011, 4, 1, 13, 37, 37, 0, tz.UTC),
            'end_time': datetime(2011, 4, 1, 13, 37, 52, 0, tz.UTC),
        }]
    }


def test_parse_model_in_geotiff_browse():
    with open(join(TEST_DATA_DIR, 'model_in_geotiff_browse.xml')) as f:
        browse_report = parse_browse_report(f)

    assert browse_report == {
        'responsible_org_name': 'DMI',
        'date_time': datetime(2012, 7, 13, 11, 54, 26, 0, tz.UTC),
        'browse_type': 'SAR',
        'browses': [{
            'type': 'model_in_geotiff_browse',
            'browse_identifier': 'ID_DODWH_MG2_CORE_09DM010001_1',
            'filename': 'ID_DEIMOS01-v2_DE0028bfp_L3R.tif',
            'image_type': 'TIFF',
            'reference_system_identifier': 'EPSG:4326',
            'start_time': datetime(2011, 2, 1, 11, 48, 1, 0, tz.UTC),
            'end_time': datetime(2011, 2, 1, 11, 48, 27, 0, tz.UTC),
        }]
    }

def test_parse_rectified_browse():
    with open(join(TEST_DATA_DIR, 'rectified_browse.xml')) as f:
        browse_report = parse_browse_report(f)

    assert browse_report == {
        'responsible_org_name': 'SLAP 03.03',
        'date_time': datetime(2014, 7, 24, 11, 58, 24, 0, tz.UTC),
        'browse_type': 'NGEO-LIGHT',
        'browses': [{
            'type': 'rectified_browse',
            'browse_identifier': 'LS05_RFUI_TM__GTC_1P_19910928T071939_19910928T072007_040292_0172_0031_B10D',
            'filename': 'http://landsat-ds.eo.esa.int/metadata/LandsatTMCloudFreeCoverage/1991/09/28/LS05_RFUI_TM__GTC_1P_19910928T071939_19910928T072007_040292_0172_0031_B10D.BP.PNG',
            'image_type': 'PNG',
            'reference_system_identifier': 'EPSG:4326',
            'rectified': {
                'coord_list': [
                    (40.8395, 40.1005),
                    (42.6645, 42.7907),
                ]
            },
            'start_time': datetime(1991, 9, 28, 7, 19, 39, 0, tz.UTC),
            'end_time': datetime(1991, 9, 28, 7, 20, 7, 0, tz.UTC),
        }]
    }


def test_parse_regular_grid_browse():
    with open(join(TEST_DATA_DIR, 'regular_grid_browse.xml')) as f:
        browse_report = parse_browse_report(f)

    assert browse_report == {
        'responsible_org_name': 'Sentinel 1 PDGS',
        'date_time': datetime(2012, 11, 8, 17, 25, 49, tzinfo=tz.tzoffset(None, 3600)),
        'browse_type': 'SAR',
        'browses': [{
            'type': 'regular_grid_browse',
            'browse_identifier': 'a20120101T043724405923',
            'filename': 'quick-look.png',
            'image_type': 'PNG',
            'reference_system_identifier': 'EPSG:4326',
            'regular_grid': {
                'col_node_number': 6,
                'row_node_number': 20,
                'col_step': 78.8,
                'row_step': 29.157894737,
                'coord_lists': [
                    [(16.80678325439503, -156.5765611873593), (16.92753830505998, -156.6004684133847), (17.04829841199142, -156.6243747867503), (17.16905739541897, -156.6482792679816), (17.28981280532396, -156.6721814810342), (17.31890450297918, -156.6779396385797)],
                    [(16.81392675836918, -156.5375764998714), (16.93467625534994, -156.561457873508), (17.05543081391954, -156.5853381897874), (17.17618425492683, -156.609216410023), (17.2969341243137, -156.6330921572468), (17.32602449992995, -156.6388438384338)],
                    [(16.82106287728486, -156.498588896317), (16.94180676422443, -156.522444395707), (17.06255571800818, -156.5462986330286), (17.1833035600995, -156.5701505703374), (17.30404783238212, -156.593999829775), (17.33313687211055, -156.599745030351)],
                    [(16.82819160727274, -156.4595983781882), (16.94892982780319, -156.4834279815255), (17.06967312036884, -156.5072561180554), (17.19041530703756, -156.5310817505579), (17.31115392562071, -156.5549045002936), (17.34024161561499, -156.5606432159938)],
                    [(16.83531294446828, -156.420604946974), (16.95604544221135, -156.4444086325018), (17.07678301711796, -156.4682106464455), (17.19751949184693, -156.4920099523122), (17.31825240012606, -156.5158061704734), (17.34733872654218, -156.5215383970218)],
                    [(16.84242688501015, -156.381608604169), (16.96315360357771, -156.4053863501777), (17.08388540437526, -156.4291622197829), (17.20461611063721, -156.4529351772322), (17.32534325199815, -156.4767048419914), (17.35442820099433, -156.4824305751009)],
                    [(16.84953342503939, -156.3426093512784), (16.97025430803387, -156.3663611361029), (17.09098027826299, -156.3901108396617), (17.21170515952094, -156.4138574269578), (17.33242647733964, -156.437600516534), (17.36151003507623, -156.4433197519077)],
                    [(16.85663256069959, -156.3036071898168), (16.97734755171454, -156.327332991834), (17.09806763490601, -156.3510565076854), (17.21878663461358, -156.3747767031367), (17.33950207225592, -156.3984931957972), (17.36858422489525, -156.4042059291287)],
                    [(16.86372428813775, -156.2646021213034), (16.98443333075823, -156.2883019189305), (17.10514747043269, -156.311999225462), (17.22586053203457, -156.3356930074195), (17.34657003285606, -156.3593828814815), (17.37565076656231, -156.3650891084553)],
                    [(16.87080860350595, -156.225594147253), (16.99151164130902, -156.2492679189448), (17.11221978097664, -156.2729389945953), (17.23292684790889, -156.2966063414503), (17.35363035525441, -156.3202695752825), (17.38270965619353, -156.3259692915751)],
                    [(16.87424257900799, -156.2066700838774), (16.994942683509, -156.2303312224714), (17.11564789218319, -156.2539895656031), (17.23635203068512, -156.2776440808492), (17.35705261014584, -156.3012943835597), (17.38613121105771, -156.3069909399195)],
                    [(16.88495498268113, -156.1475694884959), (17.00564584155052, -156.1711911438214), (17.12634181169133, -156.1948096932411), (17.24703671955967, -156.2184241052198), (17.36772806994446, -156.2420339939122), (17.39680446385366, -156.2477206758413)],
                    [(16.89201703884214, -156.1085528066608), (17.01270172358132, -156.1321483716202), (17.13339152417947, -156.1557406258028), (17.25408026763801, -156.1793285380803), (17.37476545451566, -156.2029117219735), (17.40384037416505, -156.2085918802064)],
                    [(16.89907166764901, -156.0695332249886), (17.01975012180706, -156.0931026781663), (17.14043369632741, -156.1166686157758), (17.26111621878105, -156.1402300068893), (17.38179518545178, -156.1637864645731), (17.41086861701329, -156.1694600947586)],
                    [(16.90611886532681, -156.0305107447066), (17.02679103244686, -156.054054064714), (17.14746832434206, -156.0775936644758), (17.26814456918872, -156.1011285129948), (17.38881725894098, -156.124658223118), (17.41788918858779, -156.1303253208992)],
                    [(16.91315862812868, -155.991485366909), (17.03382445174815, -156.0150025323825), (17.15449540445841, -156.0385157730854), (17.27516531508931, -156.0620240576098), (17.39583167119955, -156.0855269988818), (17.42490208510597, -156.0911875598966)],
                    [(16.92019095234127, -155.9524570925252), (17.04085037599229, -155.9759480821245), (17.161514932945, -155.9994349426222), (17.28217845274488, -156.0229166417812), (17.40283841847734, -156.046392792973), (17.43190730281881, -156.0520468128543)],
                    [(16.92721583429081, -155.9134259222858), (17.04786880150054, -155.9368907146913), (17.16852690610996, -155.9603511739047), (17.28918397845739, -155.9838062663549), (17.40983749706388, -156.0072556063005), (17.43890483801674, -156.0129030806771)],
                    [(16.93423327034943, -155.8743918566859), (17.05487972464038, -155.8978304305973), (17.17553132030729, -155.9212644675158), (17.29618188857495, -155.9446929319396), (17.41682890329453, -155.9681154395373), (17.44589468703602, -155.973756364034)],
                    [(16.94103455650875, -155.8365177408673), (17.06167463789954, -155.8599308569188), (17.18231986439887, -155.8833392385752), (17.30296406819755, -155.9067418477961), (17.42360471850083, -155.9301383022012), (17.45266897820548, -155.9357728677357)],
                ],
            },
            'start_time': datetime(2012, 1, 1, 4, 37, 24, 405923),
            'end_time': datetime(2012, 1, 1, 4, 37, 32, 890783),
        }]
    }
