from os.path import join, dirname
import logging.config
import json

import click
import yaml
import jsonschema

from .preprocess import preprocess_file, preprocess_browse
from .daemon import run_daemon
from .config import load_config


def setup_logging(debug=False):
    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'brief': {
                'format': '%(levelname)s %(name)s: %(message)s'
            }
        },
        'handlers': {
            'console': {
                'class': 'logging.StreamHandler',
                'level': 'DEBUG' if debug else 'INFO',
                'formatter': 'brief',
            }
        },
        'root': {
            'handlers': ['console'],
            'level': 'DEBUG' if debug else 'INFO',
        }
    })


def validate_config(config):
    with open(join(dirname(__file__), 'config-schema.yaml')) as f:
        schema = yaml.load(f)

    jsonschema.validate(config, schema)


@click.group()
def cli():
    pass


@cli.command(help='Run the preprocess daemon, attaching to a Redis queue')
@click.option('--config-file', type=click.File('r'))
@click.option('--use-dir', type=str) # TODO: check dir
@click.option('--validate/--no-validate', default=False)
@click.option('--host', type=str)
@click.option('--port', type=int)
@click.option('--listen-queue', type=str)
@click.option('--listen-md-queue', type=str)
@click.option('--write-queue', type=str)
@click.option('--debug/--no-debug', default=False)
def daemon(config_file=None, use_dir=None, validate=False, host=None, port=None, listen_queue=None, listen_md_queue=None, write_queue=None, debug=False):
    setup_logging(debug)
    config = load_config(config_file)
    if validate:
        validate_config(config)
    run_daemon(config, host, port, listen_queue, listen_md_queue, write_queue)


@cli.command(help='Run a single, one-off preprocessing')
@click.argument('file_path', type=str)
@click.option('--config-file', type=click.File('r'))
@click.option('--use-dir', type=str) # TODO: check dir
@click.option('--validate/--no-validate', default=False)
@click.option('--browse-report/--no-browse-report', default=False)
@click.option('--debug/--no-debug', default=False)
def preprocess(file_path, config_file=None, use_dir=None, validate=False, browse_report=False, debug=False):
    setup_logging(debug)
    config = load_config(config_file)
    if validate:
        validate_config(config)

    if browse_report:
        with open(file_path) as f:
            browse_report_data = json.load(f)

        browse_type = browse_report_data['browse_type']
        for browse in browse_report_data['browses']:
            preprocess_browse(config, browse_type, browse_report_data, browse, use_dir)
    else:
        preprocess_file(config, file_path, use_dir)

if __name__ == '__main__':
    cli()
